package pwc.taxtech.atms.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import pwc.taxtech.atms.common.AuthUserHelper;
import pwc.taxtech.atms.common.POIUtil;
import pwc.taxtech.atms.common.util.StringUtil;
import pwc.taxtech.atms.constant.enums.BackLogEnum;
import pwc.taxtech.atms.dao.OrganizationMapper;
import pwc.taxtech.atms.dao.PermissionMapper;
import pwc.taxtech.atms.dao.UserOrganizationMapper;
import pwc.taxtech.atms.dpo.*;
import pwc.taxtech.atms.dto.ApiResultDto;
import pwc.taxtech.atms.dto.backlogManage.*;
import pwc.taxtech.atms.dto.didiFileUpload.DidiFileIUploadParam;
import pwc.taxtech.atms.dto.didiFileUpload.DidiFileUploadDetailResult;
import pwc.taxtech.atms.entity.Permission;
import pwc.taxtech.atms.entity.UserOrganization;
import pwc.taxtech.atms.entity.UserOrganizationExample;
import pwc.taxtech.atms.exception.ApplicationException;
import pwc.taxtech.atms.exception.Exceptions;
import pwc.taxtech.atms.vat.dao.*;
import pwc.taxtech.atms.vat.entity.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLDecoder;
import java.text.SimpleDateFormat;
import java.time.*;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

@Service
public class BacklogManageService extends BaseService {

    @Resource
    private BacklogTypeMapper backlogTypeMapper;

    @Resource
    private BacklogGroupMapper backlogGroupMapper;

    @Resource
    private BacklogDetailMapper backlogDetailMapper;

    @Resource
    private BacklogConfigCommonMapper backlogConfigCommonMapper;

    @Resource
    private BacklogConfigMapper backlogConfigMapper;

    @Resource
    private BacklogLogMapper backlogLogMapper;

    @Resource
    private DidiFileUploadService didiFileUploadService;

    @Resource
    private PermissionMapper permissionMapper;

    @Autowired
    OrganizationServiceImpl organizationService;

    @Autowired
    private DistributedIdService idService;

    @Autowired
    private AuthUserHelper authUserHelper;

    private static final int CODE_COUNT = 8;

    private static final String GROUP_CODE_START = "G";

    private static final String CONFIG_CODE_START = "T";

    public List<OrgSelectDto> getCurrentOrgIds(String code) {
        List<Permission> permissions = permissionMapper.queryUserPermissions(authUserHelper.getCurrentUserId());
        List<String> codeList = permissions.stream().map(o -> o.getCode()).collect(Collectors.toList());
        if (codeList.contains(code)) {
            return organizationService.getMyOrgList();
        }
        return null;
    }


    public List<BacklogTypeResult> queryBacklogType(boolean getFullName) {
        BacklogTypeExample example = new BacklogTypeExample();
        example.createCriteria().andEnableEqualTo(true);
        List<BacklogType> typeList = backlogTypeMapper.selectByExample(example);
        List<BacklogTypeResult> typeResultList = new ArrayList<>();
        for (BacklogType backlogType : typeList) {
            BacklogTypeResult typeResult = new BacklogTypeResult();
            typeResult.setId(backlogType.getId());
            typeResult.setParentId(backlogType.getParentId());
            typeResult.setEnable(backlogType.getEnable());
            typeResult.setLevel(backlogType.getLevel());
            typeResult.setName(backlogType.getName());
            typeResult.setColor(backlogType.getColor());
            if (getFullName) {
                typeResult.setFullName(assembleFullName(typeList, backlogType, typeResult.getName()));
            }
            typeResultList.add(typeResult);
        }
        return typeResultList;
    }

    public String assembleFullName(List<BacklogType> typeList, BacklogType sourceResult, String fullName) {
        for (BacklogType backlogType : typeList) {
            if (!sourceResult.getParentId().equals(new Long(0)) && sourceResult.getParentId().equals(backlogType.getId())) {
                if (!fullName.equals(backlogType.getName())) {
                    fullName = backlogType.getName() + "-" + fullName;
                }

                assembleFullName(typeList, backlogType, fullName);
            }
        }
        return fullName;
    }

    public List<BacklogLogResult> queryBacklogLog(BacklogLogQueryParam param) {
        List<BacklogLogDto> dataList = backlogLogMapper.queryBacklogLog(param.getDetailId(), null);
        return dataList.stream()
                .map(o -> {
                    BacklogLogResult detailResult = new BacklogLogResult();
                    beanUtil.copyProperties(o, detailResult);
                    detailResult.setCurrUserFlag((authUserHelper.getCurrentUserId().equals(o.getPresentDealId()) ||
                            authUserHelper.getCurrentUserId().equals(o.getDealUserId())) && o.getCurrentFlag());
                    if (StringUtils.isNotBlank(o.getFileUploadIds())) {
                        String[] strings = o.getFileUploadIds().split(",");
                        DidiFileIUploadParam fileParam = new DidiFileIUploadParam();
                        fileParam.setUuids(Arrays.asList(strings));
                        PageInfo<DidiFileUploadDetailResult> uploadDetail = didiFileUploadService.queryPage(fileParam);
                        detailResult.setFileUploads(uploadDetail.getList());
                    }
                    return detailResult;
                }).collect(Collectors.toList());
    }

    public List<BacklogCalendarResult> queryBacklogCalendar(BacklogDetailQueryParam param) {
        BackLogDetailParamDto queryParam = new BackLogDetailParamDto();
        beanUtil.copyProperties(param, queryParam);
        queryParam.setCurrentSysUserId(authUserHelper.getCurrentUserId());
        List<OrgSelectDto> orgSelectDtoList = getCurrentOrgIds(BackLogEnum.AdminPermission.BACKLOG_DETAIL.getCode());
        if (CollectionUtils.isNotEmpty(orgSelectDtoList)) {
            queryParam.setAdminFlag(true);
            if (CollectionUtils.isNotEmpty(param.getOrgIdList())) {
                queryParam.setOrgIdList(orgSelectDtoList.stream().map(o -> o.getId()).collect(Collectors.toList()));
            }
            if (CollectionUtils.isNotEmpty(param.getBusinessIdList())) {
                queryParam.setBusinessIdList(orgSelectDtoList.stream().map(o -> o.getBusinessUnitId()).distinct().collect(Collectors.toList()));
            }
        }

        List<BacklogDetailDto> dataList = backlogDetailMapper.queryBacklogDetail(queryParam);
        Map<Date, Map<Long, List<BacklogDetailResult>>> dataMap = new HashMap<>();
        for (BacklogDetailDto detailDto : dataList) {
            ZoneId zone = ZoneId.systemDefault();
            LocalDateTime startDate = LocalDateTime.ofInstant(detailDto.getChinaBacklogStartDate().toInstant(), zone);
            LocalDateTime endDate = LocalDateTime.ofInstant(detailDto.getChinaBacklogEndDate().toInstant(), zone);
            for (int i = startDate.getDayOfMonth(); i <= endDate.getDayOfMonth(); i++) {
                LocalDateTime localDateTime = LocalDateTime.of(startDate.getYear(), startDate.getMonth(), i, 0, 0);
                Date localDate = Date.from(localDateTime.atZone(ZoneId.systemDefault()).toInstant());
                if (dataMap.get(localDate) == null) {
                    dataMap.put(localDate, new HashMap<>());
                }
                if (!dataMap.get(localDate).containsKey(detailDto.getCommonId())) {
                    dataMap.get(localDate).put(detailDto.getCommonId(), new ArrayList<BacklogDetailResult>());
                }
                BacklogDetailResult detailResult = new BacklogDetailResult();
                beanUtil.copyProperties(detailDto, detailResult);
                dataMap.get(localDate).get(detailDto.getCommonId()).add(detailResult);
            }
        }
        List<BacklogCalendarResult> resultList = new ArrayList<>();
        Iterator<Date> iterator = dataMap.keySet().iterator();
        while (iterator.hasNext()) {
            BacklogCalendarResult result = new BacklogCalendarResult();
            Date date = iterator.next();
            result.setDay(date);
            result.setDetailMap(dataMap.get(date));
            resultList.add(result);
        }
        return resultList;
    }

    public List<BacklogCalendarViewResult> queryBacklogCalendarView(BacklogDetailQueryParam param) {
        BackLogDetailParamDto queryParam = new BackLogDetailParamDto();
        beanUtil.copyProperties(param, queryParam);
        queryParam.setCurrentSysUserId(authUserHelper.getCurrentUserId());
        List<BacklogDetailDto> dataList = backlogDetailMapper.queryBacklogDetail(queryParam);
        Map<Long, List<BacklogDetailDto>> mapData = new HashMap<>();
        for (BacklogDetailDto detailDto : dataList) {
            if (!mapData.containsKey(detailDto.getCommonId())) {
                mapData.put(detailDto.getCommonId(), new ArrayList<>());
            }
            int size = mapData.get(detailDto.getCommonId()).size();
            if (size == 0) {
                mapData.get(detailDto.getCommonId()).add(detailDto);
            }
            boolean isAdd = false;
            for (int i = 0; i < size; i++) {
                BacklogDetailDto subDetail = mapData.get(detailDto.getCommonId()).get(i);
                if (detailDto.getChinaBacklogStartDate().compareTo(subDetail.getChinaBacklogStartDate()) < 0) {
                    isAdd = false;
                    subDetail.setChinaBacklogStartDate(detailDto.getChinaBacklogStartDate());
                    if (detailDto.getChinaBacklogEndDate().compareTo(subDetail.getChinaBacklogEndDate()) > 0) {
                        subDetail.setChinaBacklogEndDate(detailDto.getChinaBacklogEndDate());
                    }
                } else {
                    if (detailDto.getChinaBacklogStartDate().compareTo(subDetail.getChinaBacklogEndDate()) > 0) {
                        isAdd = true;
                    } else {
                        isAdd = false;
                        subDetail.setChinaBacklogEndDate(detailDto.getChinaBacklogEndDate());
                    }
                }
            }
            if (isAdd) {
                mapData.get(detailDto.getCommonId()).add(detailDto);
            }
        }
        Iterator<Long> iterator = mapData.keySet().iterator();
        Set<BacklogCalendarViewResult> viewResults = new HashSet<>();
        while (iterator.hasNext()) {
            List<BacklogDetailDto> details = mapData.get(iterator.next());
            for (BacklogDetailDto detailDto : details) {
                BacklogCalendarViewResult viewResult = new BacklogCalendarViewResult();
                viewResult.setCommonId(detailDto.getCommonId());
                viewResult.setCommonName(detailDto.getCommonName());
                viewResult.setBacklogTypeId(detailDto.getBacklogTypeId());
                viewResult.setStartDate(detailDto.getChinaBacklogStartDate());
                viewResult.setEndDate(detailDto.getChinaBacklogEndDate());
                viewResults.add(viewResult);
            }
        }
        return new ArrayList<>(viewResults);
    }

    public List<BacklogDetailResult> queryBacklogDetailForGroup(BacklogDetailQueryParam param) {
        BackLogDetailParamDto queryParam = new BackLogDetailParamDto();
        beanUtil.copyProperties(param, queryParam);
        List<OrgSelectDto> orgSelectDtoList = getCurrentOrgIds(BackLogEnum.AdminPermission.BACKLOG_DETAIL.getCode());
        queryParam.setCurrentSysUserId(authUserHelper.getCurrentUserId());
        if (CollectionUtils.isNotEmpty(orgSelectDtoList)) {
            queryParam.setAdminFlag(true);
            if (StringUtils.isNotBlank(param.getOrgId())) {
                queryParam.setOrgIdList(orgSelectDtoList.stream().map(o -> o.getId()).collect(Collectors.toList()));
            }
            if (StringUtils.isNotBlank(param.getBusinessId())) {
                queryParam.setBusinessIdList(orgSelectDtoList.stream().map(o -> o.getBusinessUnitId()).distinct().collect(Collectors.toList()));
            }
        }
        List<BacklogDetailDto> dataList = backlogDetailMapper.queryBacklogDetailForGroup(queryParam);
        return dataList.stream()
                .map(o -> {
                    BacklogDetailResult detailResult = new BacklogDetailResult();
                    beanUtil.copyProperties(o, detailResult);
                    return detailResult;
                }).collect(Collectors.toList());
    }

    public PageInfo<BacklogDetailResult> queryBacklogDetail(BacklogDetailQueryParam param) {
        List<Long> typeIds = null;
        if (param.getBacklogTypeParentId() != null) {
            BacklogTypeExample typeExample = new BacklogTypeExample();
            typeExample.createCriteria().andParentIdEqualTo(param.getBacklogTypeParentId());
            List<BacklogType> typeList = backlogTypeMapper.selectByExample(typeExample);
            typeIds = typeList.stream().map(o -> o.getId()).collect(Collectors.toList());
            param.setBacklogTypeIds(typeIds);
        }
        Page page = null;
        if (param.getPageInfo() != null && param.getPageInfo().getPageSize() != null && param.getPageInfo().getPageIndex() != null) {
            page = PageHelper.startPage(param.getPageInfo().getPageIndex(), param.getPageInfo().getPageSize());
        }
        BackLogDetailParamDto queryParam = new BackLogDetailParamDto();
        beanUtil.copyProperties(param, queryParam);
        queryParam.setCurrentSysUserId(authUserHelper.getCurrentUserId());
        List<BacklogDetailDto> dataList = backlogDetailMapper.queryBacklogDetail(queryParam);
        PageInfo<BacklogDetailResult> pageInfo = new PageInfo<>(dataList.stream()
                .map(o -> {
                    BacklogDetailResult detailResult = new BacklogDetailResult();
                    beanUtil.copyProperties(o, detailResult);
//                    if (StringUtils.isNotBlank(o.getFileUploadIds())) {
//                        String[] strings = o.getFileUploadIds().split(",");
//                        if (strings.length > 0) {
//                            DidiFileIUploadParam fileParam = new DidiFileIUploadParam();
//                            fileParam.setUuids(Arrays.asList(strings));
//                            PageInfo<DidiFileUploadDetailResult> uploadDetail = didiFileUploadService.queryPage(fileParam);
//                            detailResult.setFileUploads(uploadDetail.getList());
//                        }
//                    }

                    if (!(BackLogEnum.DetailStatus.COMPLETED.getCode().equals(detailResult.getBacklogStatus()) ||
                            BackLogEnum.DetailStatus.VOIDED.getCode().equals(detailResult.getBacklogStatus()))) {
                        LocalDate today = LocalDate.now();
                        Instant instant = detailResult.getBacklogEndDate().toInstant();
                        ZoneId zone = ZoneId.systemDefault();
                        LocalDateTime localDateTime = LocalDateTime.ofInstant(instant, zone);
                        LocalDate oldDate = localDateTime.toLocalDate();
                        Long days = ChronoUnit.DAYS.between(today, oldDate);
                        if (days > 3) {
                            detailResult.setOutTimeFlag(1);
                        } else if (days >= 1 && days <= 3) {
                            detailResult.setOutTimeFlag(2);
                        } else if (days < 1) {
                            detailResult.setOutTimeFlag(3);
                        }
                    } else {
                        detailResult.setOutTimeFlag(0);
                    }
                    return detailResult;
                }).collect(Collectors.toList()));
        if (page != null) {
            pageInfo.setTotal(page.getTotal());
        }
        return pageInfo;
    }

    public List<BacklogConfigCommonResult> querySelectedBacklogConfigCommon() {
        BacklogConfigCommonExample example = new BacklogConfigCommonExample();
        example.createCriteria().andCreateByEqualTo(authUserHelper.getCurrentUserId()).andStatusEqualTo(BackLogEnum.ConfigCommonStatus.COMPLETED.getCode());
        example.or(example.createCriteria().andUpdateByEqualTo(authUserHelper.getCurrentUserId()).andStatusEqualTo(BackLogEnum.ConfigCommonStatus.COMPLETED.getCode()));
        List<BacklogConfigCommon> dataList = backlogConfigCommonMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(dataList)) {
            return new ArrayList<>();
        } else {
            return dataList.stream()
                    .map(o -> {
                        BacklogConfigCommonResult commonResult = new BacklogConfigCommonResult();
                        beanUtil.copyProperties(o, commonResult);
                        return commonResult;
                    }).collect(Collectors.toList());
        }
    }

    public PageInfo<BacklogConfigCommonResult> queryBacklogConfigCommon(BacklogConfigCommonQueryParam param) {
        List<Long> typeIds = null;
        if (param.getBacklogTypeParentId() != null) {
            BacklogTypeExample typeExample = new BacklogTypeExample();
            typeExample.createCriteria().andParentIdEqualTo(param.getBacklogTypeParentId());
            List<BacklogType> typeList = backlogTypeMapper.selectByExample(typeExample);
            typeIds = typeList.stream().map(o -> o.getId()).collect(Collectors.toList());
        }
        Page page = PageHelper.startPage(param.getPageInfo().getPageIndex(), param.getPageInfo().getPageSize());
        BacklogConfigCommonExample example = new BacklogConfigCommonExample();
        BacklogConfigCommonExample.Criteria criteria = example.createCriteria();
        if (StringUtils.isNotBlank(param.getName())) {
            criteria.andNameLike("%" + param.getName() + "%");
        }
        if (CollectionUtils.isNotEmpty(typeIds)) {
            criteria.andBacklogTypeIdIn(typeIds);
        }
        if (param.getBacklogTypeId() != null) {
            criteria.andBacklogTypeIdEqualTo(param.getBacklogTypeId());
        }
        if (param.getAutoFlag() != null) {
            criteria.andAutoFlagEqualTo(param.getAutoFlag());
        }
        if (StringUtils.isNotBlank(param.getStatus())) {
            criteria.andStatusEqualTo(param.getStatus());
        }
        if (param.getGenerateType() != null) {
            criteria.andGenerateTypeEqualTo(param.getGenerateType());
        }
        BackLogConfigCommonParamDto queryParam = new BackLogConfigCommonParamDto();
        beanUtil.copyProperties(param, queryParam);
        if (CollectionUtils.isNotEmpty(typeIds)) {
            queryParam.setBacklogTypeIds(typeIds);
        }
        List<BacklogConfigCommon> dataList = new ArrayList<>();
        if (StringUtils.isNotBlank(param.getOrgId()) || StringUtils.isNotBlank(param.getConfigStatus()) ||
                StringUtils.isNotBlank(param.getDealUser()) || StringUtils.isNotBlank(param.getCreateUser()) ||
                StringUtils.isNotBlank(param.getAuditUser()) || StringUtils.isNotBlank(param.getCode()) ||
                StringUtils.isNotBlank(param.getGroupCode()) || StringUtils.isNotBlank(param.getBusinessLineId())) {
            dataList = backlogConfigCommonMapper.queryBacklogConfigCommon(queryParam);
        } else {
            dataList = backlogConfigCommonMapper.selectByExample(example);
        }
        PageInfo<BacklogConfigCommonResult> pageInfo = new PageInfo<>(dataList.stream()
                .map(o -> {
                    BacklogConfigCommonResult commonResult = new BacklogConfigCommonResult();
                    beanUtil.copyProperties(o, commonResult);
                    return commonResult;
                }).collect(Collectors.toList()));
        pageInfo.setTotal(page.getTotal());
        return pageInfo;
    }

    public PageInfo<BacklogConfigResult> queryBacklogConfig(BacklogConfigQueryParam param) {
        List<Long> typeIds = null;
        if (param.getBacklogTypeParentId() != null) {
            BacklogTypeExample typeExample = new BacklogTypeExample();
            typeExample.createCriteria().andParentIdEqualTo(param.getBacklogTypeParentId());
            List<BacklogType> typeList = backlogTypeMapper.selectByExample(typeExample);
            typeIds = typeList.stream().map(o -> o.getId()).collect(Collectors.toList());
            param.setBacklogTypeIds(typeIds);
        }
        Page page = null;
        if (param.getPageInfo() != null && param.getPageInfo().getPageSize() != null && param.getPageInfo().getPageIndex() != null) {
            page = PageHelper.startPage(param.getPageInfo().getPageIndex(), param.getPageInfo().getPageSize());
        }
        BackLogConfigParamDto queryParam = new BackLogConfigParamDto();
        beanUtil.copyProperties(param, queryParam);
        List<BacklogConfigDto> dataList = backlogConfigMapper.queryBacklogConfig(queryParam);
        PageInfo<BacklogConfigResult> pageInfo = new PageInfo<>(dataList.stream()
                .map(o -> {
                    BacklogConfigResult commonResult = new BacklogConfigResult();
                    beanUtil.copyProperties(o, commonResult);
                    if (StringUtils.isNotBlank(o.getFileUploadIds())) {
                        String[] strings = o.getFileUploadIds().split(",");
                        DidiFileIUploadParam fileParam = new DidiFileIUploadParam();
                        fileParam.setUuids(Arrays.asList(strings));
                        PageInfo<DidiFileUploadDetailResult> uploadDetail = didiFileUploadService.queryPage(fileParam);
                        commonResult.setFileUploads(uploadDetail.getList());
                    }
                    return commonResult;
                }).collect(Collectors.toList()));
        if (page != null) {
            pageInfo.setTotal(page.getTotal());
        }
        return pageInfo;
    }

    public PageInfo<BacklogGroupResult> queryBacklogGroup(BacklogGroupQueryParam param) {
        Page page = PageHelper.startPage(param.getPageInfo().getPageIndex(), param.getPageInfo().getPageSize());
        BackLogGroupParamDto queryParam = new BackLogGroupParamDto();
        beanUtil.copyProperties(param, queryParam);
        List<BacklogGroupDto> dataList = backlogGroupMapper.queryBacklogGroup(queryParam);
        PageInfo<BacklogGroupResult> pageInfo = new PageInfo<>(dataList.stream()
                .map(o -> {
                    BacklogGroupResult groupResult = new BacklogGroupResult();
                    beanUtil.copyProperties(o, groupResult);
                    return groupResult;
                }).collect(Collectors.toList()));
        pageInfo.setTotal(page.getTotal());
        return pageInfo;
    }

    public PageInfo<BacklogGroupResult> queryBacklogGroupDetail(BacklogGroupQueryParam param) {
        Page page = PageHelper.startPage(param.getPageInfo().getPageIndex(), param.getPageInfo().getPageSize());
        BackLogGroupParamDto queryParam = new BackLogGroupParamDto();
        beanUtil.copyProperties(param, queryParam);
        List<BacklogGroupDto> dataList = backlogGroupMapper.queryBacklogGroupDetail(queryParam);
        PageInfo<BacklogGroupResult> pageInfo = new PageInfo<>(dataList.stream()
                .map(o -> {
                    BacklogGroupResult groupResult = new BacklogGroupResult();
                    beanUtil.copyProperties(o, groupResult);
                    return groupResult;
                }).collect(Collectors.toList()));
        pageInfo.setTotal(page.getTotal());
        return pageInfo;
    }

    @Transactional
    public ApiResultDto disableBacklog(BacklogDisableParam param) {
        List<String> successList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(param.getConfigIds()) ||
                CollectionUtils.isNotEmpty(param.getDetailIds()) ||
                CollectionUtils.isNotEmpty(param.getGroupDetails())) {
            BacklogDetail backlogDetail = new BacklogDetail();
            backlogDetail.setStatus(BackLogEnum.DetailStatus.VOIDED.getCode());
            backlogDetail.setUpdateBy(authUserHelper.getCurrentUserId());
            backlogDetail.setUpdateTime(new Date());
            BacklogDetailExample detailExample = new BacklogDetailExample();
            List<Long> detailIds = null;
            if (CollectionUtils.isNotEmpty(param.getDetailIds())) {
                detailExample.createCriteria().
                        andStatusNotEqualTo(BackLogEnum.DetailStatus.COMPLETED.getCode()).
                        andCreateUserEqualTo(authUserHelper.getCurrentUserId()).
                        andIdIn(param.getDetailIds());
                List<BacklogDetail> detailList = backlogDetailMapper.selectByExample(detailExample);
                backlogDetailMapper.updateByExampleSelective(backlogDetail, detailExample);
                detailIds = new ArrayList<>();
                List<Long> configIds = new ArrayList<>();
                for (BacklogDetail detail : detailList) {
                    detailIds.add(detail.getId());
                    if (param.getDetailIds().contains(detail.getId()) && !detail.getStatus().equals(BackLogEnum.DetailStatus.VOIDED.getCode())) {
                        configIds.add(detail.getConfigId());
                    }
                }
                if (CollectionUtils.isNotEmpty(configIds)) {
                    BacklogConfigExample configExample = new BacklogConfigExample();
                    configExample.createCriteria().andIdIn(configIds);
                    List<BacklogConfig> configList = backlogConfigMapper.selectByExample(configExample);
                    successList = configList.stream().map(o -> o.getCode()).collect(Collectors.toList());
                }
            } else if (CollectionUtils.isNotEmpty(param.getConfigIds())) {
                detailExample.createCriteria().
                        andStatusNotEqualTo(BackLogEnum.DetailStatus.COMPLETED.getCode()).
                        andCreateUserEqualTo(authUserHelper.getCurrentUserId()).
                        andConfigIdIn(param.getConfigIds());
                List<BacklogDetail> detailList = backlogDetailMapper.selectByExample(detailExample);
                if (CollectionUtils.isNotEmpty(detailList)) {
                    backlogDetailMapper.updateByExampleSelective(backlogDetail, detailExample);
                    detailIds = detailList.stream().map(o -> o.getId()).collect(Collectors.toList());
                }
                BacklogConfigExample configExample = new BacklogConfigExample();
                configExample.createCriteria().andIdIn(param.getConfigIds()).andCreateUserEqualTo(authUserHelper.getCurrentUserId());
                BacklogConfig config = new BacklogConfig();
                config.setUpdateBy(authUserHelper.getCurrentUserId());
                config.setUpdateTime(new Date());
                config.setStatus(BackLogEnum.ConfigStatus.VOIDED.getCode());
                backlogConfigMapper.updateByExampleSelective(config, configExample);
            } else if (CollectionUtils.isNotEmpty(param.getGroupDetails())) {
                detailIds = new ArrayList<>();
                for (BacklogGroupDetailParam detailParam : param.getGroupDetails()) {
                    BacklogDetailQueryParam queryParam = new BacklogDetailQueryParam();
                    queryParam.setBacklogGroupId(detailParam.getGroupId());
                    queryParam.setMonth(detailParam.getMonth());
                    PageInfo<BacklogDetailResult> detailResultPageInfo = queryBacklogDetail(queryParam);
                    if (CollectionUtils.isNotEmpty(detailResultPageInfo.getList())) {
                        detailIds.addAll(detailResultPageInfo.getList().stream().map(o -> o.getId()).collect(Collectors.toList()));
                    }
                }
                if (CollectionUtils.isNotEmpty(detailIds)) {
                    detailExample.createCriteria().
                            andStatusNotEqualTo(BackLogEnum.DetailStatus.COMPLETED.getCode()).
                            andIdIn(detailIds);
                    backlogDetailMapper.updateByExampleSelective(backlogDetail, detailExample);
                }

            }
            if (CollectionUtils.isNotEmpty(detailIds)) {
                BacklogLogExample logExample = new BacklogLogExample();
                logExample.createCriteria().andDetailIdIn(detailIds);
                BacklogLog backlogLog = new BacklogLog();
                backlogLog.setCurrentFlag(false);
                backlogLog.setUpdatetime(new Date());
                backlogLog.setUpdateBy(authUserHelper.getCurrentUserId());
                backlogLogMapper.updateByExampleSelective(backlogLog, logExample);
            }
        }
        ApiResultDto apiResultDto = ApiResultDto.success(successList);
        return apiResultDto;
    }

    public ApiResultDto editBacklogConfigCommon(BacklogConfigCommonEditParam editParam) {
        BacklogConfigCommon configCommon = new BacklogConfigCommon();
        beanUtil.copyProperties(editParam, configCommon);
        configCommon.setUpdateBy(authUserHelper.getCurrentUserId());
        configCommon.setUpdateTime(new Date());
        if (editParam.getId() == null) {
            configCommon.setCreateBy(authUserHelper.getCurrentUserId());
            configCommon.setCreateTime(new Date());
            configCommon.setId(idService.nextId());
            if (StringUtils.isBlank(editParam.getStatus())) {
                editParam.setStatus(BackLogEnum.ConfigCommonStatus.UN_COMPLETED.getCode());
            }
            backlogConfigCommonMapper.insert(configCommon);
        } else {
            configCommon.setCreateTime(null);
            backlogConfigCommonMapper.updateByPrimaryKeySelective(configCommon);
        }
        ApiResultDto apiResultDto = ApiResultDto.success();
        return apiResultDto;
    }

    @Transactional
    public ApiResultDto editBacklogDetailStatus(BacklogDetailStatusEditParam editParam) {
        if (editParam.getId() == null && CollectionUtils.isEmpty(editParam.getIds())) {
            return ApiResultDto.fail();
        }
        BacklogLogExample logExample = new BacklogLogExample();
        BacklogLogExample.Criteria logCriteria = logExample.createCriteria();
        BacklogDetailExample detailExample = new BacklogDetailExample();
        BacklogDetailExample.Criteria detailCriteria = detailExample.createCriteria();
        if (editParam.getId() != null) {
            logCriteria.andDetailIdEqualTo(editParam.getId());
            detailCriteria.andIdEqualTo(editParam.getId());
        }
        if (CollectionUtils.isNotEmpty(editParam.getIds())) {
            logCriteria.andDetailIdIn(editParam.getIds());
            detailCriteria.andIdIn(editParam.getIds());
        }

        String userType = null;
        BacklogDetail record = new BacklogDetail();
        //置为失效
        if (BackLogEnum.DetailStatus.VOIDED.getCode().equals(editParam.getStatus())) {
            detailCriteria.andStatusIn(Arrays.asList(BackLogEnum.DetailStatus.UN_BEGIN.getCode(), BackLogEnum.DetailStatus.COMPLETED.getCode()));
            record.setStatus(BackLogEnum.DetailStatus.VOIDED.getCode());
        }
        //置为退回修正
        if (BackLogEnum.DetailStatus.REJECT.getCode().equals(editParam.getStatus())) {
            detailCriteria.andStatusEqualTo(BackLogEnum.DetailStatus.UN_AUDIT.getCode());
            record.setStatus(BackLogEnum.DetailStatus.REJECT.getCode());
            userType = BackLogEnum.UserType.DEAL_USER.getCode();
        }
        //置为进行中
        if (BackLogEnum.DetailStatus.BEGIN.getCode().equals(editParam.getStatus())) {
            detailCriteria.andStatusEqualTo(BackLogEnum.DetailStatus.UN_BEGIN.getCode());
            record.setStatus(BackLogEnum.DetailStatus.BEGIN.getCode());
            userType = BackLogEnum.UserType.DEAL_USER.getCode();
        }
        //置为已完成
        if (BackLogEnum.DetailStatus.COMPLETED.getCode().equals(editParam.getStatus())) {
            BacklogDetail backlogDetail = backlogDetailMapper.selectByPrimaryKey(editParam.getId());
            if (backlogDetail != null) {
                BacklogConfig backlogConfig = backlogConfigMapper.selectByPrimaryKey(backlogDetail.getConfigId());
                if (backlogConfig != null) {
                    //如果无审批人则直接完成事项，有则判断是进入审批还是完成事项
                    if (StringUtils.isBlank(backlogConfig.getAuditUser())) {
                        detailCriteria.andStatusIn(Arrays.asList(BackLogEnum.DetailStatus.BEGIN.getCode(), BackLogEnum.DetailStatus.REJECT.getCode()));
                        record.setStatus(BackLogEnum.DetailStatus.COMPLETED.getCode());
                        userType = "";
                    } else {
                        //判断当前状态是进行中还是待审核，进行中则进入待审核，待审核则进入完成
                        if (BackLogEnum.DetailStatus.BEGIN.getCode().equals(backlogDetail.getStatus()) ||
                                BackLogEnum.DetailStatus.REJECT.getCode().equals(backlogDetail.getStatus())) {
                            detailCriteria.andStatusIn(Arrays.asList(BackLogEnum.DetailStatus.BEGIN.getCode(),
                                    BackLogEnum.DetailStatus.REJECT.getCode()));
                            record.setStatus(BackLogEnum.DetailStatus.UN_AUDIT.getCode());
                            userType = BackLogEnum.UserType.AUDIT_USER.getCode();
                        } else if (BackLogEnum.DetailStatus.UN_AUDIT.getCode().equals(backlogDetail.getStatus())) {
                            //判断审批人是否是最后一位
                            BacklogLogExample lastExample = new BacklogLogExample();
                            lastExample.createCriteria().andDetailIdEqualTo(editParam.getId()).
                                    andDealRoleCodeEqualTo(BackLogEnum.UserType.AUDIT_USER.getCode()).andCurrentFlagEqualTo(true);
                            List<BacklogLog> lastLogList = backlogLogMapper.selectByExample(lastExample);
                            if (lastLogList != null && lastLogList.size() == 1) {
                                detailCriteria.andStatusEqualTo(BackLogEnum.DetailStatus.UN_AUDIT.getCode());
                                record.setStatus(BackLogEnum.DetailStatus.COMPLETED.getCode());
                                userType = "";
                            } else {
                                BacklogLogExample auditExample = new BacklogLogExample();
                                auditExample.createCriteria().andDetailIdEqualTo(editParam.getId()).
                                        andDealRoleCodeEqualTo(BackLogEnum.UserType.AUDIT_USER.getCode()).
                                        andDealUserIdEqualTo(authUserHelper.getCurrentUserId());
                                BacklogLog auditLog = new BacklogLog();
                                auditLog.setCurrentFlag(false);
                                backlogLogMapper.updateByExampleSelective(auditLog, auditExample);
                            }
                        }
                    }
                }
            }
        }

        record.setUpdateTime(new Date());
        record.setUpdateBy(authUserHelper.getCurrentUserId());
        backlogDetailMapper.updateByExampleSelective(record, detailExample);
        if (userType != null) {
            //先把把所有current_flag置为false
            BacklogLog log = new BacklogLog();
            log.setCurrentFlag(false);
            backlogLogMapper.updateByExampleSelective(log, logExample);
            if (BackLogEnum.UserType.MAPPING.containsKey(userType)) {
                logCriteria.andDealRoleCodeEqualTo(userType);
                log.setCurrentFlag(true);
                backlogLogMapper.updateByExampleSelective(log, logExample);
            }
        }
        ApiResultDto apiResultDto = ApiResultDto.success();
        return apiResultDto;
    }

    public ApiResultDto editBacklogLog(BacklogLogEditParam editParam) {
        BacklogLogExample logExample = new BacklogLogExample();
        logExample.createCriteria().andDetailIdEqualTo(editParam.getDetailId()).
                andDealUserIdEqualTo(authUserHelper.getCurrentUserId()).
                andCurrentFlagEqualTo(true);
        BacklogLog newModel = new BacklogLog();
        newModel.setCreateTime(null);
        newModel.setUpdatetime(new Date());
        newModel.setUpdateBy(authUserHelper.getCurrentUserId());
        newModel.setComment(editParam.getComment());
        newModel.setFileUploadIds(editParam.getFileUploadIds().replaceFirst(",", ""));
        backlogLogMapper.updateByExampleSelective(newModel, logExample);
        if (StringUtils.isNotBlank(editParam.getStatus())) {
            BacklogDetailStatusEditParam statusEditParam = new BacklogDetailStatusEditParam();
            statusEditParam.setId(editParam.getDetailId());
            statusEditParam.setStatus(editParam.getStatus());
            editBacklogDetailStatus(statusEditParam);

        }
        ApiResultDto apiResultDto = ApiResultDto.success();
        return apiResultDto;
    }

    @Transactional
    public ApiResultDto editBacklogConfigGroup(BacklogConfigEditListParam editParam) {
        backlogConfigMapper.clearGroup(editParam.getGroupId());
        backlogDetailMapper.clearGroup(editParam.getGroupId());
        if (CollectionUtils.isNotEmpty(editParam.getParamList())) {
            for (BacklogConfigEditParam param : editParam.getParamList()) {
                BacklogConfig backlogConfig = new BacklogConfig();
                backlogConfig.setId(param.getId());
                backlogConfig.setGroupId(param.getGroupId());
                backlogConfig.setGroupIndex(param.getGroupIndex());
                backlogConfig.setUpdateBy(authUserHelper.getCurrentUserId());
                backlogConfig.setUpdateTime(new Date());
                backlogConfigMapper.updateByPrimaryKeySelective(backlogConfig);
                BacklogDetail backlogDetail = new BacklogDetail();
                backlogDetail.setGroupId(param.getGroupId());
                backlogDetail.setGroupIndex(param.getGroupIndex());
                backlogDetail.setUpdateBy(authUserHelper.getCurrentUserId());
                backlogDetail.setUpdateTime(new Date());
                BacklogDetailExample detailExample = new BacklogDetailExample();
                detailExample.createCriteria().andConfigIdEqualTo(param.getId()).andStatusEqualTo(BackLogEnum.DetailStatus.UN_BEGIN.getCode());
                backlogDetailMapper.updateByExampleSelective(backlogDetail, detailExample);
            }
        }
        ApiResultDto apiResultDto = ApiResultDto.success();
        return apiResultDto;
    }

    public ApiResultDto editBacklogGroup(BackloGroupEditParam editParam) {
        BacklogGroup record = new BacklogGroup();
        beanUtil.copyProperties(editParam, record);
        record.setUpdateBy(authUserHelper.getCurrentUserId());
        record.setUpdateTime(new Date());
        if (editParam.getId() == null) {
            record.setCreateUser(authUserHelper.getCurrentUserId());
            record.setCreateBy(authUserHelper.getCurrentUserId());
            record.setCreateTime(new Date());
            if (StringUtils.isBlank(record.getStatus())) {
                record.setStatus(BackLogEnum.ConfigCommonStatus.UN_COMPLETED.getCode());
            }
            int insertCount = backlogGroupMapper.insertByIncrement(record);
            if (insertCount > 0 && record.getId() != null) {
                record.setGroupCode(generateCode(GROUP_CODE_START, record.getId()));
                backlogGroupMapper.updateByPrimaryKeySelective(record);
            }
        } else {
            backlogGroupMapper.updateByPrimaryKeySelective(record);
        }
        ApiResultDto apiResultDto = ApiResultDto.success();
        return apiResultDto;
    }

    public String generateCode(String start, Long id) {
        for (int i = 0; i <= CODE_COUNT - String.valueOf(id).length(); i++) {
            start += "0";
        }
        start += id;
        return start;
    }

    @Transactional
    public ApiResultDto editBacklogConfig(BacklogConfigEditParam param) {
        BacklogConfig editParam = new BacklogConfig();
        beanUtil.copyProperties(param, editParam);
        editParam.setUpdateBy(authUserHelper.getCurrentUserId());
        editParam.setUpdateTime(new Date());
        if (StringUtils.isNotBlank(editParam.getDealUser())) {
            editParam.setDealUser(editParam.getDealUser().replaceFirst(",", ""));
        }
        if (StringUtils.isNotBlank(editParam.getAuditUser())) {
            editParam.setAuditUser(editParam.getAuditUser().replaceFirst(",", ""));
        }
        if (StringUtils.isNotBlank(editParam.getFileUploadIds())) {
            editParam.setFileUploadIds(editParam.getFileUploadIds().replaceFirst(",", ""));
        }
        if (editParam.getId() == null) {
            editParam.setCreateBy(authUserHelper.getCurrentUserId());
            editParam.setCreateTime(new Date());
            if (StringUtils.isBlank(editParam.getStatus())) {
                editParam.setStatus(BackLogEnum.ConfigStatus.UN_COMPLETED.getCode());
            }
            int insertCount = backlogConfigMapper.insertByIncrement(editParam);
            if (insertCount > 0 && editParam.getId() != null) {
                editParam.setCode(generateCode(CONFIG_CODE_START, editParam.getId()));
                backlogConfigMapper.updateByPrimaryKeySelective(editParam);
            }
        } else {
            backlogConfigMapper.updateByPrimaryKeySelective(editParam);
        }
        if (BackLogEnum.ConfigStatus.COMPLETED.getCode().equals(editParam.getStatus())) {

            BacklogConfigCommonExample commonExample = new BacklogConfigCommonExample();
            commonExample.createCriteria().andIdEqualTo(editParam.getCommonId());
            List<BacklogConfigCommon> backlogConfigCommon = backlogConfigCommonMapper.selectByExample(commonExample);
            try {
                if (BackLogEnum.GenerateType.ONCE.getCode().equals(backlogConfigCommon.get(0).getGenerateType())) {
                    generateBacklogConfig(editParam);
                } else if (BackLogEnum.GenerateType.CYCLE.getCode().equals(backlogConfigCommon.get(0).getGenerateType())) {
                    if (StringUtils.isNotBlank(editParam.getWarnMonth())) {
                        Date validityStartDate = editParam.getValidityStartDate();
                        Date validityEndDate = editParam.getValidityEndDate();
                        JSONArray array = JSONArray.parseArray(editParam.getWarnMonth());
                        Instant startInstant = validityStartDate.toInstant();
                        LocalDateTime startLocalDateTime = LocalDateTime.ofInstant(startInstant, ZoneId.systemDefault());
                        Instant endInstant = validityEndDate.toInstant();
                        LocalDateTime endLocalDateTime = LocalDateTime.ofInstant(endInstant, ZoneId.systemDefault());
                        String[] endMinute = editParam.getWarnEndTime().split(":");
                        String[] startMinute = editParam.getWarnStartTime().split(":");
                        //判断有效期区间是否当年
                        if (startLocalDateTime.getYear() == endLocalDateTime.getYear()) {
                            for (int i = 0; i < array.size(); i++) {
                                int month = array.getInteger(i);
                                if (month < startLocalDateTime.getMonthValue() ||
                                        month > endLocalDateTime.getMonthValue()) {
                                    continue;
                                }
                                LocalDateTime startDate = LocalDateTime.of(startLocalDateTime.getYear(),
                                        month,
                                        Integer.valueOf(editParam.getWarnStartDay()),
                                        Integer.valueOf(startMinute[0]),
                                        Integer.valueOf(startMinute[1]));
                                editParam.setValidityStartDate(Date.from(startDate.atZone(ZoneId.systemDefault()).toInstant()));
                                LocalDateTime endDate = LocalDateTime.of(endLocalDateTime.getYear(),
                                        month,
                                        Integer.valueOf(editParam.getWarnEndDay()),
                                        Integer.valueOf(endMinute[0]),
                                        Integer.valueOf(endMinute[1]));
                                editParam.setValidityEndDate(Date.from(endDate.atZone(ZoneId.systemDefault()).toInstant()));
                                generateBacklogConfig(editParam);
                            }
                        } else {
                            for (int year = startLocalDateTime.getYear(); year <= endLocalDateTime.getYear(); year++) {
                                for (int i = 0; i < array.size(); i++) {
                                    int month = array.getInteger(i);
                                    if (year == startLocalDateTime.getYear() && month < startLocalDateTime.getMonthValue()) {
                                        continue;
                                    }
                                    if (year == endLocalDateTime.getYear() && month > endLocalDateTime.getMonthValue()) {
                                        continue;
                                    }
                                    LocalDateTime startDate = LocalDateTime.of(year,
                                            month,
                                            Integer.valueOf(editParam.getWarnStartDay()),
                                            Integer.valueOf(startMinute[0]),
                                            Integer.valueOf(startMinute[1]));
                                    editParam.setValidityStartDate(Date.from(startDate.atZone(ZoneId.systemDefault()).toInstant()));
                                    LocalDateTime endDate = LocalDateTime.of(year,
                                            month,
                                            Integer.valueOf(editParam.getWarnEndDay()),
                                            Integer.valueOf(endMinute[0]),
                                            Integer.valueOf(endMinute[1]));
                                    editParam.setValidityEndDate(Date.from(endDate.atZone(ZoneId.systemDefault()).toInstant()));
                                    generateBacklogConfig(editParam);
                                }
                            }
                        }
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
            }

        }
        ApiResultDto apiResultDto = ApiResultDto.success();
        return apiResultDto;
    }

    private void generateBacklogConfig(BacklogConfig editParam) throws Exception {
        Instant startInstant = editParam.getValidityStartDate().toInstant();
        LocalDateTime startLocalDateTime = LocalDateTime.ofInstant(startInstant, ZoneId.of(editParam.getTimeZone()));
        ZonedDateTime startZonedDateTime = startLocalDateTime.atZone(ZoneId.of("Asia/Shanghai"));
        Instant endInstant = editParam.getValidityEndDate().toInstant();
        LocalDateTime endLocalDateTime = LocalDateTime.ofInstant(endInstant, ZoneId.of(editParam.getTimeZone()));

        ZonedDateTime endZonedDateTime = endLocalDateTime.atZone(ZoneId.of("Asia/Shanghai"));
        BacklogDetail backlogDetail = new BacklogDetail();
        backlogDetail.setConfigId(editParam.getId());
        backlogDetail.setFileUploadIds(editParam.getFileUploadIds());
        backlogDetail.setGroupId(editParam.getGroupId());
        backlogDetail.setGroupIndex(editParam.getGroupIndex());
        backlogDetail.setTaxAgency(editParam.getTaxAgency());
        backlogDetail.setAmount(editParam.getAmount());
        backlogDetail.setTimeZone(editParam.getTimeZone());
        backlogDetail.setDealUser(editParam.getDealUser());
        backlogDetail.setAuditUser(editParam.getAuditUser());
        backlogDetail.setCreateUser(editParam.getCreateUser());
        backlogDetail.setCreateBy(authUserHelper.getCurrentUserId());
        backlogDetail.setCreateTime(new Date());
        backlogDetail.setOrgId(editParam.getOrgId());
        backlogDetail.setBusinessLineId(editParam.getBusinessLineId());
        backlogDetail.setId(idService.nextId());
        backlogDetail.setStatus(BackLogEnum.DetailStatus.UN_BEGIN.getCode());
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        backlogDetail.setWarnStartTime(editParam.getValidityStartDate());
        backlogDetail.setWarnEndTime(editParam.getValidityEndDate());
        backlogDetail.setChinaWarnStartTime(format.parse(startZonedDateTime.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"))));
        backlogDetail.setChinaWarnEndTime(format.parse(endZonedDateTime.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"))));
        backlogDetail.setComment(editParam.getComment());
        backlogDetailMapper.insert(backlogDetail);
        //创建log表
        BacklogLog createLog = new BacklogLog();
        createLog.setCreateBy(authUserHelper.getCurrentUserId());
        createLog.setCreateTime(new Date());
        createLog.setDetailId(backlogDetail.getId());
        createLog.setId(idService.nextId());
        createLog.setPresentDealId(editParam.getCreateUser());
        createLog.setFileUploadIds(editParam.getFileUploadIds());
        createLog.setDealRoleCode(BackLogEnum.UserType.CREATE_USER.getCode());
        createLog.setComment(editParam.getComment());
        createLog.setDealUserId(editParam.getCreateUser());
        createLog.setCurrentFlag(false);
        backlogLogMapper.insert(createLog);
        BacklogLog dealLog = new BacklogLog();
        dealLog.setCurrentFlag(true);
        dealLog.setCreateBy(authUserHelper.getCurrentUserId());
        dealLog.setCreateTime(new Date());
        dealLog.setDetailId(backlogDetail.getId());
        dealLog.setId(idService.nextId());
        dealLog.setDealRoleCode(BackLogEnum.UserType.DEAL_USER.getCode());
        dealLog.setDealUserId(editParam.getDealUser());
        dealLog.setCurrentFlag(true);
        backlogLogMapper.insert(dealLog);
        if (StringUtils.isNotBlank(editParam.getAuditUser())) {
            String[] auditArry = editParam.getAuditUser().split(",");
            for (String auditUser : auditArry) {
                if (StringUtils.isNotBlank(auditUser)) {
                    BacklogLog auditLog = new BacklogLog();
                    auditLog.setCreateBy(authUserHelper.getCurrentUserId());
                    auditLog.setCreateTime(new Date());
                    auditLog.setDetailId(backlogDetail.getId());
                    auditLog.setId(idService.nextId());
                    auditLog.setCurrentFlag(false);
                    auditLog.setDealRoleCode(BackLogEnum.UserType.AUDIT_USER.getCode());
                    auditLog.setDealUserId(auditUser);
                    backlogLogMapper.insert(auditLog);
                }
            }
        }
    }

    /**
     * 下载文件并打成zip包
     *
     * @param zos
     * @param fileUploadIds
     */
    public void downloadFileForZip(ZipOutputStream zos, List<String> fileUploadIds) {
        if (CollectionUtils.isEmpty(fileUploadIds)) {
            return;
        }
        DidiFileIUploadParam fileParam = new DidiFileIUploadParam();
        fileParam.setUuids(fileUploadIds);
        PageInfo<DidiFileUploadDetailResult> uploadDetail = didiFileUploadService.queryPage(fileParam);

        //新建一个空集合,记录下载的名字,并用于判断是否重名,重名则修改名字为(n)
        for (DidiFileUploadDetailResult item : uploadDetail.getList()) {
            String urlPath = item.getViewHttpUrl();
            InputStream is = null;
            BufferedInputStream in = null;
            byte[] buffer = new byte[1024];
            int len;
            //创建zip实体（一个文件对应一个ZipEntry）
            ZipEntry entry = new ZipEntry(item.getFileName());
            try {
                //获取需要下载的文件流
                URL httpurl = new URL(URLDecoder.decode(urlPath, "UTF-8"));
                HttpURLConnection httpConn = (HttpURLConnection) httpurl.openConnection();
                httpConn.setDoOutput(true);// 使用 URL 连接进行输出
                httpConn.setDoInput(true);// 使用 URL 连接进行输入
                httpConn.setUseCaches(false);// 忽略缓存
                httpConn.setRequestMethod("GET");// 设置URL请求方法
                //可设置请求头
                httpConn.setRequestProperty("Content-Type", "application/octet-stream");
                httpConn.setRequestProperty("Connection", "Keep-Alive");// 维持长连接
                httpConn.setRequestProperty("Charset", "UTF-8");
                httpConn.connect();
                if (httpConn.getResponseCode() >= 400) {
                    is = httpConn.getErrorStream();
                } else {
                    is = httpConn.getInputStream();
                }
                in = new BufferedInputStream(is);
                zos.putNextEntry(entry);
                //文件流循环写入ZipOutputStream
                while ((len = in.read(buffer)) != -1) {
                    zos.write(buffer, 0, len);
                }
            } catch (Exception e) {
            } finally {
                try {
                    if (zos != null) {
                        zos.closeEntry();
                    }
                    if (in != null) {
                        in.close();
                    }
                    if (is != null) {
                        is.close();
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }

    public void downloadFile(HttpServletResponse response, List<String> fileUploadIds) {
        DidiFileIUploadParam fileParam = new DidiFileIUploadParam();
        fileParam.setUuids(fileUploadIds);
        PageInfo<DidiFileUploadDetailResult> uploadDetail = didiFileUploadService.queryPage(fileParam);
        if (CollectionUtils.isEmpty(uploadDetail.getList())) {
            return;
        }
        OutputStream outputStream = null;
        CloseableHttpResponse httpResponse = null;
        CloseableHttpClient client = HttpClients.createDefault();
        InputStream in = null;
        ByteArrayOutputStream swapStream = null;
        try {
            HttpGet httpGet = new HttpGet(uploadDetail.getList().get(0).getViewHttpUrl());
            httpResponse = client.execute(httpGet);
            if (httpResponse.getStatusLine().getStatusCode() == 200) {
                in = httpResponse.getEntity().getContent();
                httpResponse.getAllHeaders();
                response.setHeader("Content-Disposition", httpResponse.getHeaders("Content-Disposition")[0].getValue());
                response.setHeader("Content-Type", httpResponse.getHeaders("Content-Type")[0].getValue());
                swapStream = new ByteArrayOutputStream();
                byte[] buff = new byte[1024];
                int rc = 0;
                while ((rc = in.read(buff, 0, 1024)) > 0) {
                    swapStream.write(buff, 0, rc);
                }
                outputStream = response.getOutputStream();
                IOUtils.write(swapStream.toByteArray(), outputStream);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (swapStream != null) {
                    swapStream.close();
                }
                if (in != null) {
                    in.close();
                }
                if (outputStream != null) {
                    outputStream.close();
                }
            } catch (IOException e) {
            }
        }
    }

    public Workbook exportBacklogDetail(InputStream is, BacklogDetailQueryParam param) {
        Workbook tWorkbook = new XSSFWorkbook();
        try {
            tWorkbook = WorkbookFactory.create(is);
            Sheet sheet = tWorkbook.getSheetAt(0);
            Row sourceRow = sheet.getRow(1);
            List<Long> typeIds = null;
            if (param.getBacklogTypeParentId() != null) {
                BacklogTypeExample typeExample = new BacklogTypeExample();
                typeExample.createCriteria().andParentIdEqualTo(param.getBacklogTypeParentId());
                List<BacklogType> typeList = backlogTypeMapper.selectByExample(typeExample);
                typeIds = typeList.stream().map(o -> o.getId()).collect(Collectors.toList());
                param.setBacklogTypeIds(typeIds);
            }
            BackLogDetailParamDto queryParam = new BackLogDetailParamDto();
            beanUtil.copyProperties(param, queryParam);
            queryParam.setCurrentSysUserId(authUserHelper.getCurrentUserId());
            List<BacklogDetailDto> dataList = backlogDetailMapper.queryBacklogDetail(queryParam);
            if (CollectionUtils.isNotEmpty(dataList)) {
                List<BacklogTypeResult> typeList = queryBacklogType(false);
                Map<Long, BacklogTypeResult> typeMap = new HashMap<>();
                for (BacklogTypeResult typeResult : typeList) {
                    typeMap.put(typeResult.getId(), typeResult);
                }

                Map<Long, List<BacklogLogDto>> logMap = new HashMap<>();
                List<BacklogLogDto> logList = backlogLogMapper.queryBacklogLog(null, dataList.stream().map(o -> o.getId()).collect(Collectors.toList()));
                for (BacklogLogDto log : logList) {
                    if (StringUtils.isBlank(log.getComment())) {
                        continue;
                    }
                    if (!logMap.containsKey(log.getDetailId())) {
                        logMap.put(log.getDetailId(), new ArrayList<>());
                    }
                    logMap.get(log.getDetailId()).add(log);
                }
                for (int i = dataList.size() - 1; i >= 0; i--) {
                    BacklogDetailDto backlogDetail = dataList.get(i);
                    Row row = POIUtil.createAndCloneRow(tWorkbook, sheet, 1, sourceRow);
                    generateBacklogDetailRow(row, backlogDetail, typeMap, logMap);

                }
            }
            sheet.removeRow(sourceRow);
            return tWorkbook;
        } catch (Exception e) {
            logger.error("export error.", e);
            throw Exceptions.SERVER_ERROR_EXCEPTION;
        }
    }

    public void generateBacklogDetailRow(Row row, BacklogDetailDto backlogDetail, Map<Long, BacklogTypeResult> typeMap, Map<Long, List<BacklogLogDto>> logMap) {
        row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_1.getIndex()).setCellValue("");
        row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_1.getIndex()).getCellStyle().setFillForegroundColor(IndexedColors.WHITE.index);
        if (!(BackLogEnum.DetailStatus.COMPLETED.getCode().equals(backlogDetail.getBacklogStatus()) ||
                BackLogEnum.DetailStatus.VOIDED.getCode().equals(backlogDetail.getBacklogStatus()))) {
            LocalDate today = LocalDate.now();
            Instant instant = backlogDetail.getBacklogEndDate().toInstant();
            ZoneId zone = ZoneId.systemDefault();
            LocalDateTime localDateTime = LocalDateTime.ofInstant(instant, zone);
            LocalDate oldDate = localDateTime.toLocalDate();
            Long days = ChronoUnit.DAYS.between(today, oldDate);
            if (days > 3) {
                row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_1.getIndex()).getCellStyle().setFillForegroundColor(IndexedColors.GREEN.index);
            } else if (days >= 1 && days <= 3) {
                row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_1.getIndex()).getCellStyle().setFillForegroundColor(IndexedColors.YELLOW.index);
            } else if (days < 1) {
                row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_1.getIndex()).getCellStyle().setFillForegroundColor(IndexedColors.RED.index);
            }
        }
        row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_2.getIndex()).setCellValue(backlogDetail.getBacklogCode());
        BacklogTypeResult typeResult = typeMap.get(backlogDetail.getBacklogTypeId());
        row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_3.getIndex()).setCellValue(typeMap.get(typeResult.getParentId()).getName());
        row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_4.getIndex()).setCellValue(typeResult.getName());
        row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_5.getIndex()).setCellValue(backlogDetail.getBacklogName());
        row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_6.getIndex()).setCellValue(backlogDetail.getBusinessLine());
        row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_7.getIndex()).setCellValue(backlogDetail.getOrgName());
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_8.getIndex()).setCellValue(format.format(backlogDetail.getChinaBacklogStartDate()));
        row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_9.getIndex()).setCellValue(format.format(backlogDetail.getChinaBacklogEndDate()));
        row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_10.getIndex()).setCellValue(format.format(backlogDetail.getBacklogStartDate()));
        row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_11.getIndex()).setCellValue(format.format(backlogDetail.getBacklogEndDate()));
        row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_12.getIndex()).setCellValue(backlogDetail.getTimeZone());
        row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_13.getIndex()).setCellValue(backlogDetail.getTaxAgency());
        if (backlogDetail.getAmount() != null) {
            row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_14.getIndex()).setCellValue(backlogDetail.getAmount().doubleValue());
        } else {
            row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_14.getIndex()).setCellValue("");
        }

        row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_15.getIndex()).setCellValue(backlogDetail.getCurrentUser());
        row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_16.getIndex()).setCellValue(BackLogEnum.DetailStatus.MAPPING.get(backlogDetail.getBacklogStatus()));
        row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_17.getIndex()).setCellValue(backlogDetail.getDealUser());
        row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_18.getIndex()).setCellValue(backlogDetail.getAuditUser());
        row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_19.getIndex()).setCellValue(backlogDetail.getCreateUser());
        row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_20.getIndex()).setCellValue(BackLogEnum.GenerateType.MAPPING.get(backlogDetail.getBacklogGenerateType()));
        row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_21.getIndex()).setCellValue(backlogDetail.getBacklogGroupCode());
        if (StringUtils.isNotBlank(backlogDetail.getFileUploadIds()) && backlogDetail.getFileUploadIds().length() > 1) {
            row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_22.getIndex()).setCellValue("是");
        } else {
            row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_22.getIndex()).setCellValue("否");
        }
        row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_23.getIndex()).setCellValue(backlogDetail.getBacklogDescribe());
        if (CollectionUtils.isNotEmpty(logMap.get(backlogDetail.getId()))) {
            String comment = "";
            for (BacklogLogDto log : logMap.get(backlogDetail.getId())) {
                comment += "[" + log.getDealUserName() + "]:" + log.getComment() + "\n";
            }
            row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_24.getIndex()).setCellValue(comment);
        } else {
            row.getCell(BackLogEnum.BacklogDetailExportColumn.Column_24.getIndex()).setCellValue("");
        }

    }

    public Workbook exportBacklogConfig(InputStream is, BacklogConfigCommonQueryParam param) {
        Workbook tWorkbook = new XSSFWorkbook();
        try {
            tWorkbook = WorkbookFactory.create(is);

            List<Long> typeIds = null;
            if (param.getBacklogTypeParentId() != null) {
                BacklogTypeExample typeExample = new BacklogTypeExample();
                typeExample.createCriteria().andParentIdEqualTo(param.getBacklogTypeParentId());
                List<BacklogType> typeList = backlogTypeMapper.selectByExample(typeExample);
                typeIds = typeList.stream().map(o -> o.getId()).collect(Collectors.toList());
            }
            BacklogConfigCommonExample example = new BacklogConfigCommonExample();
            BacklogConfigCommonExample.Criteria criteria = example.createCriteria();
            if (StringUtils.isNotBlank(param.getName())) {
                criteria.andNameLike(param.getName());
            }
            if (CollectionUtils.isNotEmpty(typeIds)) {
                criteria.andBacklogTypeIdIn(typeIds);
            }
            if (param.getBacklogTypeId() != null) {
                criteria.andBacklogTypeIdEqualTo(param.getBacklogTypeId());
            }
            if (param.getAutoFlag() != null) {
                criteria.andAutoFlagEqualTo(param.getAutoFlag());
            }
            if (StringUtils.isNotBlank(param.getStatus())) {
                criteria.andStatusEqualTo(param.getStatus());
            }
            if (param.getGenerateType() != null) {
                criteria.andGenerateTypeEqualTo(param.getGenerateType());
            }
            BackLogConfigCommonParamDto queryParam = new BackLogConfigCommonParamDto();
            beanUtil.copyProperties(param, queryParam);
            if (CollectionUtils.isNotEmpty(typeIds)) {
                queryParam.setBacklogTypeIds(typeIds);
            }
            List<BacklogConfigCommon> dataList = new ArrayList<>();
            if (StringUtils.isNotBlank(param.getOrgId()) || StringUtils.isNotBlank(param.getConfigStatus()) ||
                    StringUtils.isNotBlank(param.getDealUser()) || StringUtils.isNotBlank(param.getCreateUser()) ||
                    StringUtils.isNotBlank(param.getAuditUser()) || StringUtils.isNotBlank(param.getCode()) ||
                    StringUtils.isNotBlank(param.getGroupCode()) || StringUtils.isNotBlank(param.getBusinessLineId())) {
                dataList = backlogConfigCommonMapper.queryBacklogConfigCommon(queryParam);
            } else {
                dataList = backlogConfigCommonMapper.selectByExample(example);
            }
            Sheet sheet = tWorkbook.getSheetAt(0);
            Row sourceRow = sheet.getRow(1);
            Sheet configSheet = tWorkbook.getSheetAt(1);
            if (CollectionUtils.isNotEmpty(dataList)) {
                List<BacklogTypeResult> typeList = queryBacklogType(false);
                Map<Long, BacklogTypeResult> typeMap = new HashMap<>();
                for (BacklogTypeResult typeResult : typeList) {
                    typeMap.put(typeResult.getId(), typeResult);
                }
                int sheetNum = 2;
                for (int i = dataList.size() - 1; i >= 0; i--) {
                    BacklogConfigCommon configCommon = dataList.get(i);
                    Row row = POIUtil.createAndCloneRow(tWorkbook, sheet, 1, sourceRow);
                    BacklogTypeResult typeResult = typeMap.get(configCommon.getBacklogTypeId());
                    row.getCell(BackLogEnum.BacklogConfigCommonExportColumn.Column_1.getIndex()).setCellValue(typeMap.get(typeResult.getParentId()).getName());
                    row.getCell(BackLogEnum.BacklogConfigCommonExportColumn.Column_2.getIndex()).setCellValue(typeResult.getName());
                    row.getCell(BackLogEnum.BacklogConfigCommonExportColumn.Column_3.getIndex()).setCellValue(configCommon.getName());
                    row.getCell(BackLogEnum.BacklogConfigCommonExportColumn.Column_4.getIndex()).setCellValue(BackLogEnum.AutoFlagType.MAPPING.get(configCommon.getAutoFlag()));
                    row.getCell(BackLogEnum.BacklogConfigCommonExportColumn.Column_5.getIndex()).setCellValue(BackLogEnum.GenerateType.MAPPING.get(configCommon.getGenerateType()));
                    row.getCell(BackLogEnum.BacklogConfigCommonExportColumn.Column_6.getIndex()).setCellValue(BackLogEnum.ConfigCommonStatus.MAPPING.get(configCommon.getStatus()));
                    BackLogConfigParamDto configParam = new BackLogConfigParamDto();
                    configParam.setOrgId(param.getOrgId());
                    configParam.setStatus(param.getConfigStatus());
                    configParam.setBusinessLineId(param.getBusinessLineId());
                    configParam.setDealUser(param.getDealUser());
                    configParam.setAuditUser(param.getAuditUser());
                    configParam.setCreateUser(param.getCreateUser());
                    configParam.setCode(param.getCode());
                    configParam.setGroupCode(param.getGroupCode());
                    configParam.setCommonId(configCommon.getId());
                    List<BacklogConfigDto> configDtoList = backlogConfigMapper.queryBacklogConfig(configParam);
                    if (CollectionUtils.isNotEmpty(configDtoList)) {

                        POIUtil.cloneSheetAndStyle(configSheet, tWorkbook.createSheet(configCommon.getName() + "-分配详情"), tWorkbook);
                        String cellFormula = "HYPERLINK(\"#'" + configCommon.getName() + "-分配详情" + "'!A1\",\"" + configCommon.getName() + "\")";
                        //添加链接
                        row.getCell(BackLogEnum.BacklogConfigCommonExportColumn.Column_3.getIndex()).setCellFormula(cellFormula);
                        CellStyle linkStyle = tWorkbook.createCellStyle();
                        Font cellFont = tWorkbook.createFont();
                        cellFont.setUnderline((byte) 1);
                        cellFont.setColor(HSSFColor.BLUE.index);
                        linkStyle.setFont(cellFont);
                        row.getCell(BackLogEnum.BacklogConfigCommonExportColumn.Column_3.getIndex()).setCellStyle(linkStyle);
                        Sheet newConfigSheet = tWorkbook.getSheetAt(sheetNum);
                        Row configSourceRow = newConfigSheet.getRow(1);
                        for (int j = configDtoList.size() - 1; j >= 0; j--) {
                            BacklogConfigDto configDto = configDtoList.get(j);
                            Row configRow = POIUtil.createAndCloneRow(tWorkbook, newConfigSheet, 1, configSourceRow);
                            configRow.getCell(BackLogEnum.BacklogConfigExportColumn.Column_1.getIndex()).setCellValue(configDto.getCode());
                            configRow.getCell(BackLogEnum.BacklogConfigExportColumn.Column_2.getIndex()).setCellValue(typeMap.get(typeResult.getParentId()).getName());
                            configRow.getCell(BackLogEnum.BacklogConfigExportColumn.Column_3.getIndex()).setCellValue(typeResult.getName());
                            configRow.getCell(BackLogEnum.BacklogConfigExportColumn.Column_4.getIndex()).setCellValue(configCommon.getName());
                            configRow.getCell(BackLogEnum.BacklogConfigExportColumn.Column_5.getIndex()).setCellValue(configDto.getBusinessLineName());
                            configRow.getCell(BackLogEnum.BacklogConfigExportColumn.Column_6.getIndex()).setCellValue(configDto.getOrgName());
                            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                            configRow.getCell(BackLogEnum.BacklogConfigExportColumn.Column_7.getIndex()).setCellValue(format.format(configDto.getValidityStartDate()));
                            configRow.getCell(BackLogEnum.BacklogConfigExportColumn.Column_8.getIndex()).setCellValue(format.format(configDto.getValidityEndDate()));
                            configRow.getCell(BackLogEnum.BacklogConfigExportColumn.Column_9.getIndex()).setCellValue(configDto.getTimeZone());
                            configRow.getCell(BackLogEnum.BacklogConfigExportColumn.Column_10.getIndex()).setCellValue(BackLogEnum.ConfigStatus.MAPPING.get(configDto.getStatus()));
                            configRow.getCell(BackLogEnum.BacklogConfigExportColumn.Column_11.getIndex()).setCellValue(configDto.getDealUserName());
                            configRow.getCell(BackLogEnum.BacklogConfigExportColumn.Column_12.getIndex()).setCellValue(configDto.getAuditUserName());
                            configRow.getCell(BackLogEnum.BacklogConfigExportColumn.Column_13.getIndex()).setCellValue(configDto.getCreateUserName());
                            configRow.getCell(BackLogEnum.BacklogConfigExportColumn.Column_14.getIndex()).setCellValue(BackLogEnum.GenerateType.MAPPING.get(configDto.getGenerateType()));
                            configRow.getCell(BackLogEnum.BacklogConfigExportColumn.Column_15.getIndex()).setCellValue(configDto.getGroupCode());
                            if (StringUtils.isNotBlank(configDto.getFileUploadIds()) && configDto.getFileUploadIds().length() > 30) {
                                configRow.getCell(BackLogEnum.BacklogConfigExportColumn.Column_16.getIndex()).setCellValue("是");
                            } else {
                                configRow.getCell(BackLogEnum.BacklogConfigExportColumn.Column_16.getIndex()).setCellValue("否");
                            }
                        }
                        sheetNum++;
                        newConfigSheet.removeRow(configSourceRow);
                    }
                }
            }
            tWorkbook.removeSheetAt(1);
            sheet.removeRow(sourceRow);
            return tWorkbook;
        } catch (Exception e) {
            logger.error("export error.", e);
            throw Exceptions.SERVER_ERROR_EXCEPTION;
        }
    }

    public Workbook exportBacklogGroup(InputStream is, BacklogGroupQueryParam param) {
        Workbook tWorkbook = new XSSFWorkbook();
        try {
            tWorkbook = WorkbookFactory.create(is);
            BackLogGroupParamDto queryParam = new BackLogGroupParamDto();
            beanUtil.copyProperties(param, queryParam);
            List<BacklogGroupDto> dataList = backlogGroupMapper.queryBacklogGroupDetail(queryParam);
            Sheet sheet = tWorkbook.getSheetAt(0);
            Row sourceRow = sheet.getRow(1);
            Sheet configSheet = tWorkbook.getSheetAt(1);
            if (CollectionUtils.isNotEmpty(dataList)) {

                int sheetNum = 2;
                for (int i = dataList.size() - 1; i >= 0; i--) {
                    BacklogGroupDto groupDto = dataList.get(i);
                    Row row = POIUtil.createAndCloneRow(tWorkbook, sheet, 1, sourceRow);
                    SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                    row.getCell(BackLogEnum.BacklogGroupExportColumn.Column_1.getIndex()).setCellValue(groupDto.getGroupCode());
                    if (StringUtils.isNotBlank(groupDto.getMonth())) {
                        row.getCell(BackLogEnum.BacklogGroupExportColumn.Column_2.getIndex()).setCellValue(groupDto.getName() + "(" + groupDto.getMonth() + ")");
                    } else {
                        row.getCell(BackLogEnum.BacklogGroupExportColumn.Column_2.getIndex()).setCellValue(groupDto.getName());
                    }

                    row.getCell(BackLogEnum.BacklogGroupExportColumn.Column_3.getIndex()).setCellValue(groupDto.getOrgName());
                    row.getCell(BackLogEnum.BacklogGroupExportColumn.Column_4.getIndex()).setCellValue(groupDto.getBusinessLine());
                    row.getCell(BackLogEnum.BacklogGroupExportColumn.Column_5.getIndex()).setCellValue(groupDto.getTimeZone());
                    if (groupDto.getChinaWarnStartTime() != null) {
                        row.getCell(BackLogEnum.BacklogGroupExportColumn.Column_6.getIndex()).setCellValue(format.format(groupDto.getChinaWarnStartTime()));
                    } else {
                        row.getCell(BackLogEnum.BacklogGroupExportColumn.Column_6.getIndex()).setCellValue("");
                    }
                    if (groupDto.getChinaWarnEndTime() != null) {
                        row.getCell(BackLogEnum.BacklogGroupExportColumn.Column_7.getIndex()).setCellValue(format.format(groupDto.getChinaWarnEndTime()));
                    } else {
                        row.getCell(BackLogEnum.BacklogGroupExportColumn.Column_7.getIndex()).setCellValue("");
                    }
                    row.getCell(BackLogEnum.BacklogGroupExportColumn.Column_8.getIndex()).setCellValue(BackLogEnum.GenerateType.MAPPING.get(groupDto.getGenerateType()));
                    row.getCell(BackLogEnum.BacklogGroupExportColumn.Column_9.getIndex()).setCellValue(groupDto.getCreateUserName());
                    if (StringUtils.isNotBlank(groupDto.getStatus())) {
                        int unBeginCount = 0;
                        int complectedCount = 0;
                        int voidedCount = 0;
                        String[] statusArray = groupDto.getStatus().split(",");
                        for (String status : statusArray) {
                            if (BackLogEnum.DetailStatus.UN_BEGIN.getCode().equals(status)) {
                                unBeginCount++;
                            }
                            if (BackLogEnum.DetailStatus.COMPLETED.getCode().equals(status)) {
                                complectedCount++;
                            }
                            if (BackLogEnum.DetailStatus.VOIDED.getCode().equals(status)) {
                                voidedCount++;
                            }
                        }
                        if (unBeginCount == statusArray.length) {
                            row.getCell(BackLogEnum.BacklogGroupExportColumn.Column_10.getIndex()).setCellValue(BackLogEnum.DetailStatus.UN_BEGIN.getName());
                        } else if (complectedCount == statusArray.length) {
                            row.getCell(BackLogEnum.BacklogGroupExportColumn.Column_10.getIndex()).setCellValue(BackLogEnum.DetailStatus.COMPLETED.getName());
                        } else if (voidedCount == statusArray.length || (complectedCount + voidedCount) == statusArray.length) {
                            row.getCell(BackLogEnum.BacklogGroupExportColumn.Column_10.getIndex()).setCellValue(BackLogEnum.DetailStatus.VOIDED.getName());
                        } else {
                            row.getCell(BackLogEnum.BacklogGroupExportColumn.Column_10.getIndex()).setCellValue(BackLogEnum.DetailStatus.BEGIN.getName());
                        }
                    } else {
                        row.getCell(BackLogEnum.BacklogGroupExportColumn.Column_10.getIndex()).setCellValue(BackLogEnum.GroupCommonStatus.COMPLETED.getCode());
                    }
                    BackLogDetailParamDto detailParam = new BackLogDetailParamDto();
                    detailParam.setCurrentSysUserId(authUserHelper.getCurrentUserId());
                    detailParam.setBacklogGroupId(groupDto.getId());
                    detailParam.setMonth(groupDto.getMonth());
                    List<BacklogDetailDto> detailDtoList = backlogDetailMapper.queryBacklogDetail(detailParam);
                    if (CollectionUtils.isNotEmpty(detailDtoList)) {
                        List<BacklogTypeResult> typeList = queryBacklogType(false);
                        Map<Long, BacklogTypeResult> typeMap = new HashMap<>();
                        for (BacklogTypeResult typeResult : typeList) {
                            typeMap.put(typeResult.getId(), typeResult);
                        }
                        String linkName = "";
                        if (StringUtils.isNotBlank(groupDto.getMonth())) {
                            linkName = groupDto.getName() + "(" + groupDto.getMonth() + ")";
                        } else {
                            linkName = groupDto.getName();
                        }

                        POIUtil.cloneSheetAndStyle(configSheet, tWorkbook.createSheet(linkName + "-进行详情"), tWorkbook);
                        String cellFormula = "HYPERLINK(\"#'" + linkName + "-进行详情" + "'!A1\",\"" + linkName + "\")";
                        //添加链接
                        row.getCell(BackLogEnum.BacklogGroupExportColumn.Column_2.getIndex()).setCellFormula(cellFormula);
                        CellStyle linkStyle = tWorkbook.createCellStyle();
                        Font cellFont = tWorkbook.createFont();
                        cellFont.setUnderline((byte) 1);
                        cellFont.setColor(HSSFColor.BLUE.index);
                        linkStyle.setFont(cellFont);
                        row.getCell(BackLogEnum.BacklogConfigCommonExportColumn.Column_2.getIndex()).setCellStyle(linkStyle);
                        Sheet newDetailSheet = tWorkbook.getSheetAt(sheetNum);
                        Row detailSourceRow = newDetailSheet.getRow(1);
                        Map<Long, List<BacklogLogDto>> logMap = new HashMap<>();
                        List<BacklogLogDto> logList = backlogLogMapper.queryBacklogLog(null, detailDtoList.stream().map(o -> o.getId()).collect(Collectors.toList()));
                        for (BacklogLogDto log : logList) {
                            if (StringUtils.isBlank(log.getComment())) {
                                continue;
                            }
                            if (!logMap.containsKey(log.getDetailId())) {
                                logMap.put(log.getDetailId(), new ArrayList<>());
                            }
                            logMap.get(log.getDetailId()).add(log);
                        }
                        for (int j = detailDtoList.size() - 1; j >= 0; j--) {
                            Row configRow = POIUtil.createAndCloneRow(tWorkbook, newDetailSheet, 1, detailSourceRow);
                            BacklogDetailDto backlogDetail = detailDtoList.get(j);
                            generateBacklogDetailRow(configRow, backlogDetail, typeMap, logMap);
                        }
                        sheetNum++;
                        newDetailSheet.removeRow(detailSourceRow);
                    }
                }
            }
            tWorkbook.removeSheetAt(1);
            sheet.removeRow(sourceRow);
            return tWorkbook;
        } catch (Exception e) {
            logger.error("export error.", e);
            throw Exceptions.SERVER_ERROR_EXCEPTION;
        }
    }
}
