//Common Bind Module Method
var bindModule = function (thisModule, controllerProvider, compileProvider, filterProvider, provide) {

    thisModule.controller = controllerProvider.register;
    thisModule.directive = compileProvider.directive;
    thisModule.filter = filterProvider.register;
    thisModule.factory = provide.factory;
    thisModule.service = provide.service;

    return thisModule;
};

// register common module for shared functionalities for all other feature modules
var commonModule = angular.module('app.common', ['pascalprecht.translate', 'ngAnimate', 'ui.grid', 'ui.grid.selection', 'ui.grid.selection', 'ui.grid.treeView', 'ui.grid.resizeColumns', 'ui.grid.grouping', 'ui.grid.exporter', 'app.config', 'ui.grid.edit', 'angularFileUpload'])
    .run(['$log', function ($log) {
        $log.debug('app.common.run()...');
    }])
    // define the script files for lazy loading and method to create depencency map for resovle config in route
    .provider('scriptDependency', ['version', function (version) {
        'use strict';
        // Here must add version parameter ?v=***,
        // to solve the problem that the bundle is not refreshed when its content changed.

        this.noPermissionPage = [{
            url: '/bundles/noPermissionPage.js?v=' + version,
            type: 'text/javascript'
        }, {
            url: '/bundles/noPermissionPage.less?v=' + version,
            type: 'text/css'
        }];

        this.vat = [{
            url: '/bundles/vat.js?v=' + version,
            type: 'text/javascript'
        }, {
            url: '/bundles/vat.css?v=' + version,
            type: 'text/css'
        }, {
            url: '/bundles/vat.less?v=' + version,
            type: 'text/css'
        }];

        this.infrastructure = [{
            url: '/bundles/infrastructure.js?v=' + version,
            type: 'text/javascript'
        }, {
            url: '/bundles/infrastructure.less?v=' + version,
            type: 'text/css'
        }];


        this.calendar = [{
            url: '/bundles/calendar.js?v=' + version,
            type: 'text/javascript'
        }, {
            url: '/bundles/calendar.css?v=' + version,
            type: 'text/css'
        }, {
            url: '/bundles/calendar.less?v=' + version,
            type: 'text/css'
        }];


this.framework = [{
            url: '/bundles/framework.less?v=' + version,
            type: 'text/css'
        }];        this.summary = [

            {
                url: '/bundles/summary.css?v=' + version,
                type: 'text/css'
            }];

        this.main = [
            {
                url: '/bundles/appMain.less?v=' + version,
                type: 'text/css'
            }];

        this.createDependenciesMap = function (dependencies) {
            if (!angular.isArray(dependencies)) {
                throw new TypeError('"scriptUrls" should be an array type!');
            }

            var dependenciesMap = {
                dependency: ['$q', '$rootScope',
                    function ($q, $rootScope) {
                        var deferred = $q.defer();
                        PWC.Loader.load(dependencies, function (hasNewJsLoaded) {
                            // If has new JavaScript resource loaded, should call $apply, otherwise,
                            // should not call it.
                            if (hasNewJsLoaded) {
                                $rootScope.$apply(function () {
                                    deferred.resolve();
                                });
                            } else {
                                deferred.resolve();
                            }
                        });
                        return deferred.promise;
                    }
                ]
            };

            return dependenciesMap;
        };

        this.$get = function () {
            return {
                createDependenciesMap: createDependenciesMap,
                infrastructure: infrastructure,
                vat: vat,
                noPermissionPageModule: noPermissionPageModule,
                calendar: calendar,
                main:main,
                framework:framework
            };
        };
    }]);

// register services module
var webservices = angular.module('app.webservices', ['app.common'])
    .run(['$log', function ($log) {
        $log.debug('app.webservices.run()...');
    }]);

// register framework module for application framework
var frameworkModule = angular.module('app.framework', ['app.webservices', 'app.common', 'ui.dashboard'])
    .run(['$log', function ($log) {
        $log.debug('app.framework.run()...');
    }])
    .config(['$controllerProvider', '$urlRouterProvider', '$compileProvider', '$filterProvider', '$provide', 'scriptDependencyProvider', '$stateProvider',
        function ($controllerProvider, $urlRouterProvider, $compileProvider, $filterProvider, $provide, scriptDependencyProvider, $stateProvider) {
            'use strict';
            // this is required to add controller/directive/filter/service after angular bootstrap
            bindModule(frameworkModule, $controllerProvider, $compileProvider, $filterProvider, $provide);

            //铃铛通知
            $stateProvider.state({
                name: 'notification',
                url: '/notification',
                views: {
                    '@': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                            function ($scope, $stateParams, appTranslation) {
                                appTranslation.load([appTranslation.appPart]);
                            }],
                        template: '<app-push-notification></app-push-notification>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
                deepStateRedirect: true,
                sticky: true
            });


            $stateProvider.state({
                name: 'summary',
                url: '/summary',
                views: {
                    '@': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                            function ($scope, $stateParams, appTranslation) {
                                appTranslation.load([appTranslation.appPart]);
                            }],
                        template: '<app-summary></app-summary>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(_.flatten([scriptDependencyProvider.summary, scriptDependencyProvider.vat])),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'mainPage',
                url: '/mainPage',
                views: {
                    '@': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                            function ($scope, $stateParams, appTranslation) {
                                appTranslation.load([appTranslation.appPart]);
                            }],
                        template: '<app-main></app-main>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(_.flatten([scriptDependencyProvider.main])),
                deepStateRedirect: true,
                sticky: true
            });
        }
    ]);

// register cache module for application framework
var cacheModule = angular.module('app.cache', ['app.common'])
    .run(['$log', 'cacheService', function ($log, cacheService) {
        $log.debug('app.cache.run()...');

    }]);

var vatModule = angular.module('app.vat', ['ui.grid', 'ui.grid.selection', 'ui.grid.moveColumns', 'ui.grid.grouping', 'ui.grid.expandable', 'ui.grid.treeView', 'ui.grid.exporter', 'ui.grid.resizeColumns', 'mc.resizer', 'ui.select', 'ngAnimate', 'ngSanitize', 'ui.bootstrap', 'ui.grid.edit', 'perfect_scrollbar', 'dx', 'ui.grid.pinning',])
    .run(['$log', function ($log) {
        $log.debug('app.vat.run()...');
    }])
    .config(['$controllerProvider', '$urlRouterProvider', '$compileProvider', '$filterProvider', '$provide', '$stateProvider', 'uiSelectConfig', 'scriptDependencyProvider',
        function ($controllerProvider, $urlRouterProvider, $compileProvider, $filterProvider, $provide, $stateProvider, uiSelectConfig, scriptDependencyProvider) {
            'use strict';
            // this is required to add controller/directive/filter/service after angular bootstrap
            bindModule(vatModule, $controllerProvider, $compileProvider, $filterProvider, $provide);

            uiSelectConfig.theme = 'select2';

            $stateProvider.state({
                name: 'vat',
                url: '/vat',
                cache: false,
                views: {
                    '@': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                            function ($scope, $stateParams, appTranslation) {
                                appTranslation.load([appTranslation.vat]);
                            }],
                        template: '<vat-layout></vat-layout>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
                deepStateRedirect: true,
                sticky: true
            });


            /**********the end of a beautiful separated line for analyzeReport*********/

            $stateProvider.state({
                name: 'vat.devGrid',
                url: '/devGrid',
                views: {
                    '@vat': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                            function ($scope, $stateParams, appTranslation) {
                                appTranslation.load([appTranslation.vat]);
                            }],
                        template: '<dev-grid></dev-grid>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
                deepStateRedirect: true,
                sticky: true
            });
        }]);

//税务日历模块
var taxCalendarModule = angular.module('app.taxCalendar', ['mc.resizer', 'ui.select', 'ngSanitize', 'perfect_scrollbar', 'dx', 'ngFileSaver'])
    .run(['$log', function ($log) {
        $log.debug('app.taxCalendar.run()...');
    }])
    .config(['$controllerProvider', '$urlRouterProvider', '$compileProvider', '$filterProvider', '$provide', '$stateProvider', 'uiSelectConfig', 'scriptDependencyProvider',
        function ($controllerProvider, $urlRouterProvider, $compileProvider, $filterProvider, $provide, $stateProvider, uiSelectConfig, scriptDependencyProvider) {
            'use strict';
            bindModule(taxCalendarModule, $controllerProvider, $compileProvider, $filterProvider, $provide);

            //税务日历
            $stateProvider.state({
                name: 'taxCalendar',
                url: '/taxCalendar',
                views: {
                    '@': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                            function ($scope, $stateParams, appTranslation) {
                                appTranslation.load([appTranslation.appPart]);
                            }],
                        template: '<calendar-display></calendar-display>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.calendar),
                deepStateRedirect: true,
                sticky: true
            });

        }]);
