webservices.factory('interactionService',
    ['$http', '$q', 'apiConfig', 'apiInterceptor', 'jqFetch',
        function ($http, $q, apiConfig, apiInterceptor, jqFetch) {
            'use strict';
            return {
                getMainList: function (params) {
                    return jqFetch.get(apiInterceptor.webApiHostUrl + '/taxEmail/listAdmin', params);
                },
                getTagList: function (type) {
                    return jqFetch.get(apiInterceptor.webApiHostUrl + '/taxEmail/getTagList?type=' + type);
                },
                addTagSubmit: function (tag, type) {
                    return jqFetch.get(apiInterceptor.webApiHostUrl + '/taxEmail/addTagSubmit?tag=' + tag + '&type=' + type);
                }
                ,
                editEmailType: function (params) {
                    return jqFetch.post(apiInterceptor.webApiHostUrl + '/taxEmail/edit', params);
                },
                exportTableData: function (params) {
                    $('#busy-indicator-container').show();
                    var defer = $q.defer();
                    var xhr = new XMLHttpRequest();
                    xhr.onload = function (e) {
                        var arraybuffer = xhr.response;
                        console.info("arraybuffer:", arraybuffer);
                        defer.resolve(arraybuffer, xhr.status, xhr.getAllResponseHeaders());
                    };
                    xhr.open("POST", apiInterceptor.webApiHostUrl + '/fileTypes/exportExcel', true);
                    xhr.setRequestHeader("Authorization", apiInterceptor.tokenType + ' ' + apiInterceptor.apiToken());
                    xhr.setRequestHeader("Content-Type", 'application/json;charset=UTF-8');
                    xhr.responseType = "arraybuffer";
                    xhr.send(JSON.stringify(params));
                    return defer.promise;
                },
                addShareHandle: function (obj) {
                    return $http.post('/taxEmail/addTaxEmail', obj, apiConfig.create());
                },
                editShareHandle: function (obj) {
                    return $http.post('/taxEmail/editTaxEmail', obj, apiConfig.create());
                },
                getShareDataById: function (id) {
                    return $http.get('/taxEmail/getShareDataById?id=' + id, apiConfig.create());
                },
                deleteShareDataById: function (id) {
                    return $http.post('/taxEmail/'+id+'/delete','', apiConfig.create());
                },
                editInternationTax: function (obj) {
                    return $http.post('/InternationSystem/editInternationSystem', obj, apiConfig.create());
                },
                addInternationTax: function (obj) {//Institution
                    return $http.post('/InternationSystem/addInternationSystem', obj, apiConfig.create());
                },
                getInternationTaxById: function (id) {
                    return $http.get('/InternationSystem/getInternationTaxById?id=' + id, apiConfig.create());
                },
                deleteInternationById: function (id) {
                    return $http.post('/InternationSystem/'+id+'/delete','', apiConfig.create());
                },
                getListData: function () {
                    return $http.get('/InternationSystem/getListData', apiConfig.create());
                },
                getRemotePdfUrl: function (id) {
                    return $http.get('/InternationSystem/getRemotePdfUrl?id=' + id, apiConfig.create());
                },
                readXLSX: function (params) {
                    return $http.post('/taxDoc/previewExcelToJson', params, apiConfig.create());
                },
                readPDF: function (params) {
                    return $http.post('/InternationSystem/previewPDF', params, apiConfig.create());
                },
                postMessageSubmit: function (obj) {
                    return $http.post('/Interactive/postMessageSubmit', obj, apiConfig.create());
                },
                postQuestion: function (obj) {
                    return $http.post('/Interactive/postQuestion', obj, apiConfig.create());
                },
                postVoteSubmit: function (obj) {
                    return $http.post('/Interactive/postVoteSubmit', obj, apiConfig.create());
                },
                showDetail: function (id) {
                    return $http.get('/Interactive/showDetail?id=' + id, apiConfig.create());
                },
                subitTicket: function (obj) {
                    return $http.post('/Interactive/subitTicket', obj, apiConfig.create());
                },
                setIntervalVote: function (id) {
                    return $http.get('/Interactive/setIntervalVote?id=' + id, apiConfig.create());
                },
                preLoadTask: function (id) {
                    var url = '/Interactive/preLoadTask';
                    if (id != undefined) {
                        url += '?id=' + id;
                    }

                    return $http.get(url, apiConfig.create());
                },
                judgeTask: function (id) {
                    return $http.get('/Interactive/judgeTask?id=' + id, apiConfig.create());
                },
                taskSubmit: function (obj) {
                    return $http.post('/Interactive/taskSubmit', obj, apiConfig.create());
                },
                getTaxManageList: function (params) {
                    return jqFetch.get(apiInterceptor.webApiHostUrl + '/InteractionTaxManage/listAdmin', params);
                },
                editTaxManage: function (obj) {
                    return $http.post('/InteractionTaxManage/edit', obj, apiConfig.create());
                },
                getTaxManageById: function (id) {
                    return $http.get('/InteractionTaxManage/' + id, apiConfig.create());
                },
                getLinkList: function (params) {
                    return jqFetch.get(apiInterceptor.webApiHostUrl + '/Link/listAdmin', params);
                },
                collaboratorSumbit : function (obj) {
                    return $http.post('/Interactive/collaboratorSumbit', obj, apiConfig.create());
                },
                getInteractiveAdminLists: function(params){
                    return $http.get('/Interactive/getInteractiveAdminLists', apiConfig.create());
                },
                deleteInteractiveAdmin : function (id) {
                    return $http.get('/Interactive/deleteInteractiveAdmin?id=' + id, apiConfig.create());
                },
                doRecommended : function(id, type){
                    return $http.get('/Interactive/doRecommended?id=' + id + '&type=' + type, apiConfig.create());
                },
                postEdit : function (data, type) {
                    return $http.post('/Interactive/postEdit?type=' + type, data, apiConfig.create());
                },
                selectById : function (id) {
                    return $http.get('/Interactive/selectById?id=' + id, apiConfig.create());
                },
                getHomeShareData : function(){
                    return $http.get('/taxEmail/getHomeShareData', apiConfig.create());
                },
                getHomeInteractiveData : function(){
                    return $http.get('/Interactive/getHomeInteractiveData', apiConfig.create());
                }
            };
        }]);
