﻿// web service proxy for voucher
webservices.factory('uploadFileService', ['$http', 'apiConfig', '$q', '$log', function ($http, apiConfig, $q, $log) {
    'use strict';
    var downloadFileHelper = function (data, status, headers, defaultFileName) {

        $('#busy-indicator-container').show();

        var defer = $q.defer();
        var octetStreamMime = 'application/octet-stream';
        var success = false;

        headers = headers();
        var filename = decodeURI(headers['x-file-name']) || defaultFileName;

        // Determine the content type from the header or default to "application/octet-stream"
        var contentType = headers['content-type'] || octetStreamMime;

        try {
            // Try using msSaveBlob if supported
            var blob = new Blob([data], { type: contentType });
            if (navigator.msSaveBlob)
                navigator.msSaveBlob(blob, filename);
            else {
                // Try using other saveBlob implementations, if available
                var saveBlob = navigator.webkitSaveBlob || navigator.mozSaveBlob || navigator.saveBlob;
                if (saveBlob === undefined) throw "Not supported";
                saveBlob(blob, filename);
            }
            success = true;
        } catch (ex) {
            $log.debug("saveBlob method failed with the following exception:");
            $log.debug(ex);
        }

        if (!success) {
            // Get the blob url creator
            var urlCreator = window.URL || window.webkitURL || window.mozURL || window.msURL;
            if (urlCreator) {
                // Try to use a download link
                var link = document.createElement('a');
                if ('download' in link) {
                    // Try to simulate a click
                    try {
                        // Prepare a blob URL
                        var blobdownload = new Blob([data], { type: contentType });
                        var urldownload = urlCreator.createObjectURL(blobdownload);
                        link.setAttribute('href', urldownload);

                        link.setAttribute("download", filename);

                        // Simulate clicking the download link
                        var event = document.createEvent('MouseEvents');
                        event.initMouseEvent('click', true, true, window, 1, 0, 0, 0, 0, false, false, false, false, 0, null);
                        link.dispatchEvent(event);
                        success = true;

                    } catch (ex) {
                        $log.debug("Download link method with simulated click failed with the following exception:");
                        $log.debug(ex);
                        $q.reject();
                    }
                }

                if (!success) {
                    try {
                        // Prepare a blob URL
                        // Use application/octet-stream when using window.location to force download
                        var blobsuccess = new Blob([data], { type: octetStreamMime });
                        var urlsuccess = urlCreator.createObjectURL(blobsuccess);
                        window.location = urlsuccess;
                        success = true;
                    } catch (ex) {
                        $log.debug(ex);
                        $q.reject();
                    }
                }
            }
        }

        if (!success) {
            // Fallback to window.open method
            $log.debug("No methods worked for saving the arraybuffer, using last resort window.open");
            window.open(httpPath, '_blank', '');
        }

        defer.resolve('success');
        $('#busy-indicator-container').hide();
        return defer.promise;
    };

    return {
        deleteFile: function (fileID) {
            return $http.get('/uploadFileToDb/deleteFile/' + fileID, apiConfig.create());
        },
        downloadFile: function (fileID, defaultFileName) {
            $http.get('/uploadFileToDb/downloadFile/' + fileID, apiConfig.create({ responseType: 'arraybuffer' }))
            .success(function (data, status, headers) {
                downloadFileHelper(data, status, headers, defaultFileName);
            });
        }
    };
}]);